//
//	Data Object Viewer
//
//	Written by J Brown 2003
//	www.catch22.net
//
#define STRICT
#include <windows.h>
#include <commctrl.h>
#include "resource.h"

#define APPNAME "IDataObject Viewer"

#pragma comment(linker, "/OPT:NOWIN98")

#pragma comment(lib, "comctl32.lib")

HWND hwndMain;
HWND hwndList;
HINSTANCE hInstance;

void RegisterDropWindow(HWND hwnd);
void UnregisterDropWindow(HWND hwnd);
BOOL EnumClipboardData(HWND hwnd);

HWND CreateListView(HWND hwndParent)
{
	HWND		hwndList;
	LVCOLUMN	lvcol;
	int			width = 512;
	
	hwndList = CreateWindowEx(WS_EX_CLIENTEDGE, WC_LISTVIEW, "", 
			WS_CHILD|WS_VISIBLE|LVS_REPORT, 0, 0, 0, 0, hwndParent, 0,
			hInstance, 0);

	width -= GetSystemMetrics(SM_CXVSCROLL);

	lvcol.pszText   = "cfFormat";
	lvcol.mask		= LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvcol.cx		= 164;
	lvcol.iSubItem  = 0;
	ListView_InsertColumn(hwndList, 0, &lvcol);
	width -= lvcol.cx;


	lvcol.pszText	= "ptd";
	lvcol.cx		= 54;
	ListView_InsertColumn(hwndList, 1, &lvcol);
	width -= lvcol.cx;

	lvcol.pszText	= "dwAspect";
	lvcol.cx		= 90;
	ListView_InsertColumn(hwndList, 2, &lvcol);
	width -= lvcol.cx;

	lvcol.pszText	= "lindex";
	lvcol.cx		= 68;
	ListView_InsertColumn(hwndList, 3, &lvcol);
	width -= lvcol.cx;

	lvcol.pszText	= "tymed";
	lvcol.cx		= max(120, width+1);
	ListView_InsertColumn(hwndList, 4, &lvcol);
	width -= lvcol.cx;

	return hwndList;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HICON hIcon;

	switch(msg)
	{
	case WM_CREATE:

		hwndList = CreateListView(hwnd);

		// set small icon for window
		hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
		SendMessage(hwnd, WM_SETICON, ICON_BIG, (LPARAM)hIcon);

		RegisterDropWindow(hwnd);
		
		return TRUE;

	case WM_ERASEBKGND:
		// stop background flicker!
		return 1;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDM_FILE_EXIT:
			CloseWindow(hwnd);
			return 0;

		case IDM_FILE_ABOUT:
			MessageBox(hwnd, "IDataObject Viewer\r\n\r\n"
							 "Copyright(c) 2003 by Catch22 Productions\t\r\n"
							 "Written by J Brown.\r\n\r\n"
							 "Homepage at www.catch22.net", APPNAME, MB_ICONINFORMATION);
			return 0;

		case IDM_FILE_CLIP:
			EnumClipboardData(hwnd);
			return 0;
		}

		break;

	case WM_CLOSE:
		// shut program down
		UnregisterDropWindow(hwnd);
		DestroyWindow(hwnd);
		PostQuitMessage(0);
		return 0;

	case WM_SIZE:
		// resize listbox to fit in main window
		MoveWindow(hwndList, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
		return 0;
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}

void InitMainWnd()
{
	WNDCLASSEX wc = { sizeof(wc) };

	wc.lpfnWndProc   = WndProc;
	wc.lpszClassName = APPNAME;
	wc.lpszMenuName  = MAKEINTRESOURCE(IDR_MENU1);
	wc.hInstance     = hInstance;

	RegisterClassEx(&wc);
}

void CreateMainWnd()
{
	hwndMain = CreateWindowEx(0, APPNAME, APPNAME, 
		WS_VISIBLE|WS_OVERLAPPEDWINDOW, 
		CW_USEDEFAULT, CW_USEDEFAULT, 512, 200, 0,0,
		hInstance, 0);
}

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmdLine, int nShowCmd)
{
	MSG msg;

	hInstance = hInst;

	// This program requires COM
	OleInitialize(0);

	InitCommonControls();

	InitMainWnd();
	CreateMainWnd();

	// message-pump
	while(GetMessage(&msg, 0, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	// Shutdown COM
	OleUninitialize();

	return 0;
}

