#define STRICT
#include <windows.h>
#include <commctrl.h>

extern "C" BOOL EnumClipboardData(HWND hwnd);
extern "C" BOOL EnumData(HWND hwnd, IDataObject *pDataObject);

#define STRING_(s) s,#s

typedef struct 
{
	UINT  cfFormat;
	char *name;

} STRLOOKUP;

STRLOOKUP cliplook[] = 
{
	STRING_(CF_BITMAP),
	STRING_(CF_DIB),
	STRING_(CF_DIBV5),
	STRING_(CF_DIF),
	STRING_(CF_DSPBITMAP),
	STRING_(CF_DSPENHMETAFILE),
	STRING_(CF_DSPMETAFILEPICT),
	STRING_(CF_DSPTEXT),
	STRING_(CF_ENHMETAFILE),
	STRING_(CF_GDIOBJFIRST),
	STRING_(CF_HDROP),
	STRING_(CF_LOCALE),
	STRING_(CF_METAFILEPICT),
	STRING_(CF_OEMTEXT),
	STRING_(CF_OWNERDISPLAY),
	STRING_(CF_PALETTE),
	STRING_(CF_PENDATA),
	STRING_(CF_PRIVATEFIRST),
	STRING_(CF_RIFF),
	STRING_(CF_SYLK),
	STRING_(CF_TEXT),
	STRING_(CF_WAVE),
	STRING_(CF_TIFF),
	STRING_(CF_UNICODETEXT),
	0,0
};

STRLOOKUP aspectlook[]  = 
{
	DVASPECT_CONTENT,   "Content  ", 
	DVASPECT_THUMBNAIL, "Thumbnail", 
	DVASPECT_ICON,      "Icon     ", 
	DVASPECT_DOCPRINT,  "DocPrint ",
	0, 0,
};

STRLOOKUP tymedlook[]  = 
{
	TYMED_HGLOBAL,  "hGlobal",
	TYMED_FILE,	    "File",
	TYMED_ISTREAM,	"IStream",
	TYMED_ISTORAGE, "IStorage",
	TYMED_GDI,		"GDI",
	TYMED_MFPICT,	"MFPict",
	TYMED_ENHMF,	"ENHMF",
	TYMED_NULL,		"Null",
	0,0,
};

char *lookupfmt(STRLOOKUP *lookup, UINT cfFormat)
{
	for(int i = 0; cliplook[i].name != 0; i++)
	{
		if(lookup[i].cfFormat == cfFormat)
		{
			return lookup[i].name;
		}
	}

	return 0;
}

/*
void AddFormatListBox(HWND hwndList, FORMATETC *pfmtetc)
{
	char ach[256];
	char namebuf[64];
	char ptd[12];
	char *name = namebuf;
	char *aspect;
	char *tymed;
	char *achptr = ach;

	// Get textual name of format
	if(!GetClipboardFormatName(pfmtetc->cfFormat, name, 64))
	{
		if((name = lookupfmt(cliplook, pfmtetc->cfFormat)) == 0)
			name = "Unknown Format";
	}

	if((aspect = lookupfmt(aspectlook, pfmtetc->dwAspect)) == 0)
		aspect = "{Unknown}  ";

	if(pfmtetc->ptd != 0)
		wsprintf(ptd, "{%08x}", pfmtetc->ptd);
	else
		lstrcpy(ptd, "{Null}");

	achptr += wsprintf(ach, "%-24s ptd=%s  dwAspect=%s  lindex={%2d}  tymed={", 
		name, 
		ptd, 
		aspect,
		pfmtetc->lindex);

	// now add all supported data mediums
	for(int i = 1; i <= 64; i <<= 2)
	{
		if(pfmtetc->tymed & i)
		{
			if((tymed = lookupfmt(tymedlook, i)) == 0)
				tymed = "Unknown";
			
			achptr += wsprintf(achptr, "%s,", tymed);
		}
	}

	if(achptr[-1] == ',') achptr--;
	lstrcpy(achptr, "}");

	SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM)ach);
}*/

void AddFormatListView(HWND hwndList, FORMATETC *pfmtetc)
{
	char tymedbuf[256] = "";
	char namebuf[64];
	char ptd[12];
	char buf[16];
	char *name = namebuf;
	char *aspect;
	char *tymed;
	char *tymedptr;

	LVITEM lvitem;
	DWORD  nIndex = ListView_GetItemCount(hwndList);

	// Get textual name of format
	if(!GetClipboardFormatName(pfmtetc->cfFormat, name, 64))
	{
		if((name = lookupfmt(cliplook, pfmtetc->cfFormat)) == 0)
			name = "Unknown Format";
	}

	lvitem.mask		= LVIF_TEXT;
	lvitem.iSubItem = 0;
	lvitem.iItem	= nIndex;
	lvitem.pszText  = name;

	// add format name
	ListView_InsertItem(hwndList, &lvitem);

	//
	//	Add TARGETDEVICE pointer
	//
	if(pfmtetc->ptd != 0)
		wsprintf(ptd, "%08x", pfmtetc->ptd);
	else
		lstrcpy(ptd, "");

	ListView_SetItemText(hwndList, nIndex, 1, ptd);

	//
	//	Add DVASPECT_xxx constant
	//
	if((aspect = lookupfmt(aspectlook, pfmtetc->dwAspect)) == 0)
		aspect = "Unknown  ";

	ListView_SetItemText(hwndList, nIndex, 2, aspect);

	//
	//	Add lindex value
	//
	wsprintf(buf, "%d", pfmtetc->lindex);
	ListView_SetItemText(hwndList, nIndex, 3, buf);

	//
	//	Add TYMED value(s)
	//
	tymedptr = tymedbuf;

	// now add all supported data mediums
	for(int i = 1; i <= 64; i <<= 2)
	{
		if(pfmtetc->tymed & i)
		{
			if((tymed = lookupfmt(tymedlook, i)) == 0)
				tymed = "Unknown";
			
			tymedptr += wsprintf(tymedptr, ", %s", tymed);
		}
	}

	ListView_SetItemText(hwndList, nIndex, 4, tymedbuf+2);
}

BOOL EnumData(HWND hwnd, IDataObject *pDataObject)
{
	FORMATETC		fmtetc;
	IEnumFORMATETC *pEnumFormats;
	HWND			hwndList;
	ULONG			num;
	
	hwndList = GetWindow(hwnd, GW_CHILD);
	
	//SendMessage(hwndList, LB_RESETCONTENT, 0, 0);
	ListView_DeleteAllItems(hwndList);

	// Get the COM interface for format enumeration
	if(S_OK != pDataObject->EnumFormatEtc(DATADIR_GET, &pEnumFormats))
		return FALSE;

	// Enumerate each type of data supported by this IDataObject, one-by-one
	while(S_OK == pEnumFormats->Next(1, &fmtetc, &num))
	{
		AddFormatListView(hwndList, &fmtetc);
	}

	pEnumFormats->Release();

	return TRUE;
}

BOOL EnumClipboardData(HWND hwnd)
{
	IDataObject *pDataObject;

	if(S_OK == OleGetClipboard(&pDataObject))
	{
		EnumData(hwnd, pDataObject);
		pDataObject->Release();
	}

	return TRUE;
}
